@@ -137,6 +137,8 @@ struct _GpModule
 
   GetAppletIdFromIidFunc   compatibility_func;
 
+  GetStandaloneMenuFunc    standalone_menu_func;
+
   GHashTable              *applets;
 };
 
@@ -555,11 +557,40 @@ gp_module_get_applet_id_from_iid (GpModule    *module,
   return module->compatibility_func (old_iid);
 }
 
+/**
+ * gp_module_set_standalone_menu:
+ * @module: a #GpModule
+ * @func: the function to call to create a menu
+ *
+ * Specifies a function to be used to create standalone menu.
+ */
+void
+gp_module_set_standalone_menu (GpModule              *module,
+                               GetStandaloneMenuFunc  func)
+{
+  module->standalone_menu_func = func;
+}
+
+GtkWidget *
+gp_module_get_standalone_menu (GpModule *module,
+                               gboolean  enable_tooltips,
+                               gboolean  locked_down,
+                               guint     menu_icon_size)
+{
+  if (module->standalone_menu_func == NULL)
+    return NULL;
+
+  return module->standalone_menu_func (enable_tooltips,
+                                       locked_down,
+                                       menu_icon_size);
+}
+
 /**
  * gp_module_applet_new:
  * @module: a #GpModule
  * @applet: the applet id
  * @settings_path: the #GSettings path to the per-instance settings
+ * @initial_settings: initial settings
  * @error: return location for a #GError, or %NULL
  *
  * Returns a newly allocated applet.
@@ -570,6 +601,7 @@ GpApplet *
 gp_module_applet_new (GpModule     *module,
                       const gchar  *applet,
                       const gchar  *settings_path,
+                      GVariant     *initial_settings,
                       GError      **error)
 {
   GpAppletInfo *info;
@@ -606,6 +638,7 @@ gp_module_applet_new (GpModule     *module,
   return g_object_new (type,
                        "id", applet,
                        "settings-path", settings_path,
+                       "initial-settings", initial_settings,
                        "gettext-domain", module->gettext_domain,
                        NULL);
 }
